﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace ELTE.Forms.BackgroundWorkerUsage
{
    public partial class WorkerForm : Form
    {
        private BackgroundWorker _Worker;

        public WorkerForm()
        {
            InitializeComponent();

            _Worker = new BackgroundWorker();
            _Worker.WorkerReportsProgress = true;
            _Worker.WorkerSupportsCancellation = true;
            _Worker.DoWork += new DoWorkEventHandler(Worker_DoWork); // párhuzamos munkavégzés
            _Worker.ProgressChanged += new ProgressChangedEventHandler(Worker_ProgressChanged); // előrehaladás
            _Worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(Worker_RunWorkerCompleted); // munka vége

            _ButtonCancel.Enabled = false;
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;
            try
            {
                Int32 factor = Convert.ToInt32(e.Argument);
                for (Int32 i = 1; i <= factor; i++)
                {
                    if (worker.CancellationPending) // ha leállítottuk a folyamatot
                    {
                        e.Cancel = true;
                        return;
                    }

                    this.Invoke(new EventHandler(delegate
                    {
                        _ListResults.Items.Insert(0, String.Format("processing {0}", i));
                    }));

                    Thread.Sleep(200);
                    worker.ReportProgress(
                        Convert.ToInt32(
                            (100 / Convert.ToSingle(factor)) * i
                        )
                    ); // előrehaladás jelentés

                }
            }
            catch (InvalidOperationException oex)
            {
                MessageBox.Show(oex.Message);
            }
        }

        private void Worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            _ProgressBar.Value = e.ProgressPercentage;
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            MessageBox.Show("Completed background task.", "BackgroundWorker Usage");
            _ButtonGo.Enabled = true;
            _ButtonCancel.Enabled = false;
        }

        private void ButtonGo_Click(object sender, EventArgs e)
        {
            _Worker.RunWorkerAsync(Convert.ToInt32(_NumericFactor.Value)); // folyamat indítás
            _ButtonGo.Enabled = false;
            _ButtonCancel.Enabled = true;
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            _Worker.CancelAsync(); // folyamat leállítás
        }
    }
}
