﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.Model.Traversal
{
    /// <summary>
    /// Szintfolytonos bejárás típusa.
    /// </summary>
    public class LevelorderTraversal<KeyType, ValueType> : TreeTraversal<KeyType, ValueType> where KeyType : IComparable<KeyType>
    {
        protected Queue<BinarySearchTree<KeyType, ValueType>.Node> _LevelQueue; // a szintfolytonos bejáráshoz sor kell

        public override Boolean Running
        {
            get { return (_LevelQueue.Count != 0); }
        }
        public override KeyType CurrentKey
        {
            get { return _VisitedNodes[_VisitedNodes.Count - 1].Key; }
        }
        public override ValueType CurrentValue
        {
            get { return _VisitedNodes[_VisitedNodes.Count - 1].Value; }
        }

        public LevelorderTraversal(BinarySearchTree<KeyType, ValueType> tree)
            : base(tree)
        {
            _LevelQueue = new Queue<BinarySearchTree<KeyType, ValueType>.Node>();
       }

        public override void Start()
        {
            _VisitedNodes.Clear();
            _LevelQueue.Clear();
            if (_Tree.Root != null)
                _LevelQueue.Enqueue(_Tree.Root); // betesszük a gyökér elemet a sorba
        }

        public override void Step()
        {
            BinarySearchTree<KeyType, ValueType>.Node node = _LevelQueue.Dequeue(); // kivesszük az első elemet
            if (node.Left != null) // ha a bal gyerek nem nulla
                _LevelQueue.Enqueue(node.Left); // felvesszük a sorba
            if (node.Right != null)
                _LevelQueue.Enqueue(node.Right);
            _VisitedNodes.Add(node); // hozzávesszük a bejárt csúcsok közé
        }

        
    }
}
