﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.Model.Traversal
{
    /// <summary>
    /// Preorder bejárás típusa.
    /// </summary>
    public class PreorderTraversal<KeyType, ValueType> : TreeTraversal<KeyType, ValueType>
        where KeyType : IComparable<KeyType>
    {
        protected Stack<BinarySearchTree<KeyType, ValueType>.Node> _LevelOrderStack; // az iterált változathoz egy vermet kell használni

        public override Boolean Running
        {
            get { return (_LevelOrderStack.Count > 0); }
        }
        public override KeyType CurrentKey
        {
            get { return _VisitedNodes[_VisitedNodes.Count - 1].Key; }
        }
        public override ValueType CurrentValue
        {
            get { return _VisitedNodes[_VisitedNodes.Count - 1].Value; }
        }

        public PreorderTraversal(BinarySearchTree<KeyType, ValueType> tree)
            : base(tree)
        {
            _LevelOrderStack = new Stack<BinarySearchTree<KeyType, ValueType>.Node>();
        }

        public override void Start()
        {
            if (_Tree.Root != null)
                _LevelOrderStack.Push(_Tree.Root);
        }

        public override void Step()
        {
            BinarySearchTree<KeyType, ValueType>.Node node = _LevelOrderStack.Pop();

            if (node.Right != null)
                _LevelOrderStack.Push(node.Right);
            if (node.Left != null)
                _LevelOrderStack.Push(node.Left);
            _VisitedNodes.Add(node);
        }
    }
}
