﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.Model.Traversal
{
    /// <summary>
    /// Általános bináris keresőfa bejárás típusa.
    /// </summary>
    public abstract class TreeTraversal<KeyType, ValueType> where KeyType : IComparable<KeyType>
    {
        protected BinarySearchTree<KeyType, ValueType> _Tree; // a bejárandó fa
        protected List<BinarySearchTree<KeyType, ValueType>.Node> _VisitedNodes; // a már bejárt csúcsok listája
        // mivel a Node már beágyazott osztály, a fán keresztül hivatkozhatunk rá

        /// <summary>
        /// Algoritmus futásának lekérdezése.
        /// </summary>
        public abstract Boolean Running { get; }
        /// <summary>
        /// Aktuális kulcs lekérdezése.
        /// </summary>
        public abstract KeyType CurrentKey { get; }
        /// <summary>
        /// Aktuális érték lekérdezése.
        /// </summary>
        public abstract ValueType CurrentValue { get; }

        public TreeTraversal(BinarySearchTree<KeyType, ValueType> tree)
        {
            _Tree = tree;
            _VisitedNodes = new List<BinarySearchTree<KeyType, ValueType>.Node>();
        }
        /// <summary>
        /// Indítás.
        /// </summary>
        public abstract void Start();
        /// <summary>
        /// Léptetés.
        /// </summary>
        public abstract void Step();
        /// <summary>
        /// Teljes futtatás.
        /// </summary>
        /// <returns>A bejárt csúcsok listája.</returns>
        public List<ValueType> Run()
        {
            List<ValueType> values = new List<ValueType>();
            if (_Tree != null)
            {
                Start();
                while (Running)
                {
                    Step();
                    values.Add(CurrentValue);
                }
            }
            return values;
        }
    }
}
