﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.Serialization.Formatters.Binary;
using ELTE.Algorithms.BinarySearchTree.Serialize.Model;
using ELTE.Algorithms.BinarySearchTree.Serialize.Model.Traversal;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.View
{
    public partial class TreeForm : Form
    {
        private VisualBinarySearchTree<Int32, String> _Tree;
        private TreeTraversal<Int32, String> _Traversal; // a sablonparamétereket int és string típussal helyettesítjük

        public TreeForm()
        {
            InitializeComponent();

            _Tree = new VisualBinarySearchTree<Int32, String>(); // új bináris keresőfa
        }

        private void ButtonSearch_Click(object sender, EventArgs e)
        {
            try
            {
                String result = _Tree.Search(Convert.ToInt32(_NumericKey.Value)); // művelet lefuttatása és eredmény lekérdezése
                _Panel.Refresh(); // panel frissítése
                _StatusLabel.Text = "Key found, value: " + result.ToString();
            }
            catch (KeyMissingException)
            {
                _StatusLabel.Text = "Key not found!";
            }
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            try
            {
                _Tree.Insert(Convert.ToInt32(_NumericKey.Value), _TextValue.Text);
                _Panel.Refresh();
                _StatusLabel.Text = "Insert succeded.";
            }
            catch (DuplicateKeyException)
            {
                _StatusLabel.Text = "Insert failed, key already in tree!";
            }
        }

        private void ButtonRemove_Click(object sender, EventArgs e)
        {
            try
            {
                _Tree.Delete(Convert.ToInt32(_NumericKey.Value));
                _Panel.Refresh();
                _StatusLabel.Text = "Delete succeded.";
            }
            catch (KeyMissingException)
            {
                _StatusLabel.Text = "Key not found!";
            }
        }

        private void ButtonMin_Click(object sender, EventArgs e)
        {
            try
            {
                _Tree.Min();
                _Panel.Refresh();
                _StatusLabel.Text = "Minimum value is: " + _Tree.Current.Value.ToString();
            }
            catch (TreeIsEmptyException)
            { 
                _StatusLabel.Text = "Tree is empty!";
            
            }
        }

        private void ButtonMax_Click(object sender, EventArgs e)
        {
            try
            {
                _Tree.Max();
                _Panel.Refresh();
                _StatusLabel.Text = "Manimum value is: " + _Tree.Current.Value.ToString();
            }
            catch (TreeIsEmptyException)
            {
                _StatusLabel.Text = "Tree is empty!";

            }
        }

        private void ButtonPrev_Click(object sender, EventArgs e)
        {
            try
            {
                _Tree.Previous();
                _Panel.Refresh();
                _StatusLabel.Text = "Previous value is: " + _Tree.Current.Value.ToString();
            }
            catch (TreeIsEmptyException)
            {
                _StatusLabel.Text = "Tree is empty!";

            }
        }

        private void ButtonNext_Click(object sender, EventArgs e)
        {
            try
            {
                _Tree.Next();
                _Panel.Refresh();
                _StatusLabel.Text = "Next value is: " + _Tree.Current.Value.ToString();
            }
            catch (TreeIsEmptyException)
            {
                _StatusLabel.Text = "Tree is empty!";
            }
        }

        private void ButtonClear_Click(object sender, EventArgs e)
        {
            _Tree.Clear();
            _Panel.Refresh();
            _StatusLabel.Text = "Tree cleared.";
        }
        private void TreePanel_Paint(object sender, PaintEventArgs e) // panel frissítésének eseménykezelője
        {
            // automatikusan fehérre festi a hátteret
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias; // élsimítás beállítása
            if (_Traversal != null && _Traversal is IDrawable) // ha a bejárás kirajzolható
                    (_Traversal as IDrawable).Draw(e.Graphics, _Panel.Width); // akkor rajzoljuk ki azt
            else
                _Tree.Draw(e.Graphics, _Panel.Width); // fa kirajzolása
        }

        private void ButtonStartStep_Click(object sender, EventArgs e)
        {
            if (_ButtonStartStep.Text == "Start")
            {
                switch (_ComboSelectTraversal.SelectedIndex)
                {
                    case 0:
                        _Traversal = new VisualLevelorderTraversal<Int32, String>(_Tree);
                        break;
                    case 1:
                        _Traversal = new VisualPreorderTraversal<Int32, String>(_Tree);
                        break;
                }
                if (_Traversal != null)
                {
                    _Traversal.Start();
                    if (_Traversal.Running)
                        _ButtonStartStep.Text = "Step";
                }
            }
            else
            {
                _Traversal.Step();
                _StatusLabel.Text = "Value " + _Traversal.CurrentValue.ToString() + " traversed.";
                if (!_Traversal.Running)
                    _ButtonStartStep.Text = "Start";
            }
            _Panel.Refresh();
        }

        private void ButtonRun_Click(object sender, EventArgs e)
        {
            String text = "Values ";
            switch (_ComboSelectTraversal.SelectedIndex)
            {
                case 0:
                    _Traversal = new LevelorderTraversal<Int32, String>(_Tree);
                    break;
                case 1:
                    _Traversal = new PreorderTraversal<Int32, String>(_Tree);
                    break;
            }
            if (_Traversal != null)
            {
                List<String> values = _Traversal.Run();
                foreach (String val in values)
                    text += val.ToString() + " ";
                text += " traversed.";

                _StatusLabel.Text = text;
            }
            _Panel.Refresh();
        }

        private void MenuFileNew_Click(object sender, EventArgs e)
        {
            _Tree = new VisualBinarySearchTree<Int32, String>();
            _Traversal = null;
            _ComboSelectTraversal.SelectedIndex = -1;
            _Panel.Refresh();
        }

        private void MenuFileLoad_Click(object sender, EventArgs e)
        {
            _OpenFileDialog.Filter = "Binary Search Tree (*.bst)|*.bst"; // a dialógusablak szűrője
            if (_OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    FileStream readStream = new FileStream(_OpenFileDialog.FileName, FileMode.Open); // fájl olvasó folyam
                    BinaryFormatter formatter = new BinaryFormatter(); // bináris formázó objektum
                    _Tree = (formatter.Deserialize(readStream) as VisualBinarySearchTree<Int32, String>); // objektum deszérializálása az adatfolyamból
                    readStream.Close();
                    _Panel.Refresh();
                }
                catch (Exception)
                {
                    _Tree.Clear();
                    MessageBox.Show("Failed to open file!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void MenuFileSave_Click(object sender, EventArgs e)
        {
            _SaveFileDialog.Filter = "Binary Search Tree (*.bst)|*.bst";
            if (_SaveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    FileStream writeStream = new FileStream(_SaveFileDialog.FileName, FileMode.OpenOrCreate); // fájl író folyam
                    BinaryFormatter formatter = new BinaryFormatter(); // bináris formázó objektum
                    formatter.Serialize(writeStream, _Tree); // objektum szérializálása az adatfolyamba
                    writeStream.Close();
                }
                catch (Exception)
                {
                    MessageBox.Show("Failed to open file!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void MenuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
