﻿using System;
using System.Drawing;
using ELTE.Algorithms.BinarySearchTree.Serialize.Model;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.View
{
    /// <summary>
    /// Kirajzolható bináris keresőfa típusa.
    /// </summary>
    /// <typeparam name="KeyType">Kulcs típusa.</typeparam>
    /// <typeparam name="ValueType">Érték típusa.</typeparam>
    [Serializable]
    public class VisualBinarySearchTree<KeyType, ValueType> : BinarySearchTree<KeyType, ValueType>, IDrawable where KeyType : IComparable<KeyType>
    {
        /// <summary>
        /// Fa kirajzolása.
        /// </summary>
        /// <param name="gr">Rajzobjektum, amivel rajzolunk.</param>
        /// <param name="width">A szélesség, amelyben rajzolunk.</param>
        public void Draw(Graphics gr, Int32 width)
        {
            if (_Root != null) // ha van elem a fában
                DrawNode(_Root, gr, 0, width, 20); // meghívunk egy rekurzív eljárást, amely egy csúcsot ki tud rajzolni
        }
        private void DrawNode(Node node, Graphics gr, Int32 start, Int32 width, Int32 height)
        // egy csúcs és hozzá tartozó élek kirajzolása, megkapja, hogy melyik csúcsot, milyen rajzobjektummal, milyen kezdőpozícióval, 
        // milyen szélességben és milyen magasságban kell kirajzolnia
        {
            Font font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold); // betűtípus
            Pen bluePen = new Pen(Color.Blue); // kék toll a csúcs és élrajzoláshoz
            Pen redPen = new Pen(Color.Red); // piros toll az aktuálisan kijelölt csúcsnak
            SizeF keySize = gr.MeasureString(node.Key.ToString(), font); // le kell mérnünk, mekkora a csúcs kiírása ahhoz, hogy középre rakhassuk

            if (node.Left != null) // ha van gyerek, akkor azt is ki kell rajzolnunk
            {
                gr.DrawLine(bluePen, start + width / 2, height, start + width / 4, height + 50);
                // él kirajzolása a két csúcs közé
                DrawNode(node.Left, gr, start, width / 2, height + 50); // rekurzívan meghívjuk az eljárást a bal gyerek csúcsra
            }
            if (node.Right != null) // ugyaanez a jobb gyerek csúcsra
            {
                gr.DrawLine(bluePen, start + width / 2, height, start + 3 * width / 4, height + 50);
                DrawNode(node.Right, gr, start + width / 2, width / 2, height + 50);
            }

            gr.FillEllipse(Brushes.White, start + width / 2 - 15, height - 15, 30, 30); // háttér újrarajzolása
            if (node == _Current) // ha az aktuális csúcsot rajzoljuk éppen, akkor pirossal
                gr.DrawEllipse(redPen, start + width / 2 - 15, height - 15, 30, 30);
            else // különben kékkel
                gr.DrawEllipse(bluePen, start + width / 2 - 15, height - 15, 30, 30);
            gr.DrawString(node.Key.ToString(), font, Brushes.Black, start + width / 2 - keySize.Width / 2, height - keySize.Height / 2); // belerajzoljuk a kulcsot
        }
    }
}
