﻿using System;
using System.Drawing;
using ELTE.Algorithms.BinarySearchTree.Serialize.Model;
using ELTE.Algorithms.BinarySearchTree.Serialize.Model.Traversal;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.View
{
    /// <summary>
    /// Kirajzolható szintfolytonos bejárás típusa.
    /// </summary>
    /// <typeparam name="KeyType">Kulcs típusa.</typeparam>
    /// <typeparam name="ValueType">Érték típusa.</typeparam>
    public class VisualLevelorderTraversal<KeyType, ValueType> : LevelorderTraversal<KeyType, ValueType>, IDrawable where KeyType : IComparable<KeyType>
    {
        public VisualLevelorderTraversal(BinarySearchTree<KeyType, ValueType> tree) : base(tree) { }

        public void Draw(Graphics gr, Int32 width)
        {
            if (_LevelQueue.Count > 0 || _VisitedNodes.Count > 0) // ha már dolgozott az eljárás
                DrawNode(_Tree.Root, gr, 0, width, 20); // a saját rajzolófüggvényeket hívjuk
            else if (_Tree is IDrawable)
                (_Tree as IDrawable).Draw(gr, width); // különben a fát rajzoljuk ki
        }

        private void DrawNode(BinarySearchTree<KeyType, ValueType>.Node node, Graphics gr, Int32 start, Int32 width, Int32 height)
        {
            Font font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold);
            Pen greenPen = new Pen(Color.Green);
            Pen redPen = new Pen(Color.Red);
            Pen blackPen = new Pen(Color.Black);
            SizeF keySize = gr.MeasureString(node.Key.ToString(), font);

            if (node.Left != null)
            {
                gr.DrawLine(greenPen, start + width / 2, height, start + width / 4, height + 50);
                DrawNode(node.Left, gr, start, width / 2, height + 50);
            }
            if (node.Right != null)
            {
                gr.DrawLine(greenPen, start + width / 2, height, start + 3 * width / 4, height + 50);
                DrawNode(node.Right, gr, start + width / 2, width / 2, height + 50);
            }

            gr.FillEllipse(Brushes.White, start + width / 2 - 15, height - 15, 30, 30);
            if (_LevelQueue.Contains(node))
                gr.DrawEllipse(redPen, start + width / 2 - 15, height - 15, 30, 30);
            else if (_VisitedNodes.Contains(node))
                gr.DrawEllipse(greenPen, start + width / 2 - 15, height - 15, 30, 30);
            else
                gr.DrawEllipse(blackPen, start + width / 2 - 15, height - 15, 30, 30);
            gr.DrawString(node.Key.ToString(), font, Brushes.Black, start + width / 2 - keySize.Width / 2, height - keySize.Height / 2); // belerajzoljuk a kulcsot
        }
    }
}
