﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ELTE.Forms.Islands
{
    public partial class MainForm : Form
    {
        private Random _RandomGenerator; // véletlenszám generátor
        private Int32[] _Heights; // magasságok

        public MainForm()
        {
            InitializeComponent();
        }

        private void Loaded(object sender, EventArgs e)
        {
            _RandomGenerator = new Random();
            _Heights = new Int32[_Panel.Width / 10];
        }

        private void GenerateMap(object sender, EventArgs e)
        {
            Int32 maxHeight = _Panel.Height / 2;
            for (Int32 i = 1; i < _Heights.Length; i++)
            {
                Int32 actHeight;
                do
                {
                    actHeight = _RandomGenerator.Next(-maxHeight, maxHeight - 40);
                    // 40-nel csökkentjük a maximum értéket, hogy kiférjen a zászló
                }
                while (Math.Abs(actHeight - _Heights[i - 1]) > 20);
                // addig generálunk, amíg az előző érték és ennek különbsége nem több 20-nál

                _Heights[i] = actHeight;
            }

            Graphics graphics = _Panel.CreateGraphics(); // rajzoló objektum

            graphics.Clear(Color.White); // fehér háttér kirajzolása
            graphics.DrawLine(Pens.Blue, 0, _Panel.Height / 2, _Panel.Width, _Panel.Height / 2);
            // tengerszint kirajzolása

            for (Int32 i = 1; i < _Heights.Length; i++)
                graphics.DrawLine(Pens.Green, 10 * (i - 1), maxHeight - _Heights[i - 1], 10 * i, maxHeight - _Heights[i]);
            // vonalak kirajzolása

        }

        private void ShowPeaks(object sender, EventArgs e)
        {
            Graphics graphics = _Panel.CreateGraphics();

            for (Int32 i = 0; i < _Heights.Length; i++)
            {
                if (IsPeak(i)) // ha csúcs
                {
                    Int32 x = 10 * i;
                    Int32 y = _Panel.Height / 2 - _Heights[i];

                    Pen pen = new Pen(Color.Red, 2);

                    // zászlót 3 vonalból húzzuk össze:
                    graphics.DrawLine(pen, x, y, x, y - 40);
                    graphics.DrawLine(pen, x, y - 40, x + 10, y - 30);
                    graphics.DrawLine(pen, x, y - 20, x + 10, y - 30);
                }
            }
        }

        private Boolean IsPeak(Int32 index) // megadja, hogy az adott pont szigetcsúcs-e
        {
            Boolean isPeak = (_Heights[index] > 0); // csúcs, ha a tenger felett van

            // sziget végét megnézzük
            Int32 j = index;
            while (j < _Heights.Length && _Heights[j] > 0)
            {
                if (_Heights[j] > _Heights[index]) // ha van nagyobb csúcs
                    isPeak = false; // akkor már nem csúcs
                j++;
            }

            // sziget elejét megnézzük:
            Int32 k = index;
            while (k >= 0 && _Heights[k] > 0)
            {
                if (_Heights[k] > _Heights[index])
                    isPeak = false;
                k--;
            }
            return isPeak;
        }

    }
}
