﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using ELTE.Forms.MultiImageDisplay.Model;

namespace ELTE.Forms.MultiImageDisplay.View
{
    public partial class MainForm : Form
    {
        private List<DisplayableImage> _LoadedImages; // képeket tartalmazó lista
        private DisplayableImage _CurrentImage; // aktuálisan kiválassztott kép
        private Int32 _StartIndex; // a megjelenített lista kezdő indexe
        private Point _LastMousePos; // utolsó egérpozíció
        private FolderBrowserDialog _FolderBrowserDialog; // könyvtár böngésző

        public MainForm()
        {
            InitializeComponent();
            Text = Application.ProductName;

            // egérkezelő műveletek a kép mozgatásához:
            _PictureBoxLargeMap.MouseDown += new MouseEventHandler(PictureBoxLargeMap_MouseDown);
            _PictureBoxLargeMap.MouseUp += new MouseEventHandler(PictureBoxLargeMap_MouseUp);
            _PictureBoxLargeMap.MouseMove += new MouseEventHandler(PictureBoxLargeMap_MouseMove);

            _FolderBrowserDialog = new FolderBrowserDialog();
            _FolderBrowserDialog.RootFolder = Environment.SpecialFolder.MyComputer; // a teljes sajátgépet látja
            _FolderBrowserDialog.ShowNewFolderButton = false; // nem lehet új könyvtárat létrehozni

            _LoadedImages = new List<DisplayableImage>();
            _StartIndex = 0;
            _NavigatorControl.Enabled = _ButtonListLeft.Enabled = _ButtonListRight.Enabled = false;
        }

        private void MenuFileOpen_Click(object sender, EventArgs e) // könyvtár megnyitása
        {
            if (_FolderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                LoadDirectory(_FolderBrowserDialog.SelectedPath);
            }
        }

        private void LoadDirectory(String path) // könyvtár betöltése
        {
            String[] fileNames = Directory.GetFiles(path); // fájlnevek begyűjtése
            foreach (String fileName in fileNames)
            {
                try
                {
                    _LoadedImages.Add(new DisplayableImage(fileName)); // új kép felvétele
                    _ImageList.Images.Add(_LoadedImages[_LoadedImages.Count - 1].Image); // kép felvétele a képek listájába
                }
                catch (Exception) { } // ha hiba történt (nem képet töltöttünk be), akkor hagyjuk figyelmen kívül
            }
            ListImages();
            _ButtonListLeft.Enabled = _ButtonListRight.Enabled = true;
        }

        private void ListImages() // képek listázása
        {
            _ListViewImages.BeginUpdate(); // frissítés kezdete, így nem változik a megjelenés
            _ListViewImages.Items.Clear();

            for (Int32 i = _StartIndex; i < _StartIndex + 5 && i < _LoadedImages.Count; i++) // 5 kép fér a listába
            {
                _ListViewImages.Items.Add(_LoadedImages[i].Title, i); // fájlnevek és képek betöltése
            }
            _ListViewImages.EndUpdate(); // frissítés vége, változhat a megjelenítés
        }

        private void MenuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // lista léptetése balra és jobbra:
        private void ButtonListLeft_Click(object sender, EventArgs e)
        {
            if (_StartIndex > 0)
            {
                _StartIndex -= 5;
                ListImages();
            }
        }
        private void ButtonListRight_Click(object sender, EventArgs e)
        {
            if (_StartIndex + 5 < _LoadedImages.Count)
            {
                _StartIndex += 5;
                ListImages();
            }
        }

        private void ListMaps_SelectedIndexChanged(object sender, EventArgs e) // kiválasztunk egy képet a listából
        {
            if (_ListViewImages.SelectedIndices.Count > 0)
            {
                _CurrentImage = _LoadedImages[_StartIndex + _ListViewImages.SelectedIndices[0]];
                _CurrentImage.Reset(); // kép visszaállítása alapállapotba
                _NavigatorControl.Reset(); // vezérlők visszaállítása alapállapotba
                _CurrentImage.FullDisplay = _PictureBoxSmallMap;
                _CurrentImage.ZoomDisplay = _PictureBoxLargeMap;
                _NavigatorControl.Enabled = true;
                _NavigatorControl.Image = _CurrentImage; // aktuális kép beállítása a vezérlőknek
            }
        }

        private void PictureBoxLargeMap_MouseMove(object sender, MouseEventArgs e) // egérmozgatás
        {
            if (_CurrentImage != null && e.Button != MouseButtons.None) // valamelyik gomb le van nyomva
            {
                _CurrentImage.Move(_LastMousePos.X - e.X, _LastMousePos.Y - e.Y); // kép mozgatása
                _LastMousePos = e.Location;
            }
        }

        private void PictureBoxLargeMap_MouseUp(object sender, MouseEventArgs e) // egér lenyomása
        {
            _PictureBoxLargeMap.Cursor = System.Windows.Forms.Cursors.Arrow; // egérkurzus beállítása
        }

        private void PictureBoxLargeMap_MouseDown(object sender, MouseEventArgs e) // egér felengedése
        {
            _PictureBoxLargeMap.Cursor = System.Windows.Forms.Cursors.SizeAll;
            _LastMousePos = e.Location; // utolsó pozíció megjegyzése
        }
    }
}
