﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using ELTE.Forms.MultiImageDisplay.Model;

namespace ELTE.Forms.MultiImageDisplay.View
{
    /// <summary>
    /// Navigációs vezérlő típusa.
    /// </summary>
    public partial class NavigatorControl : UserControl
    {
        private DisplayableImage _Image; // megjeleníthető kép
        private Int32 _ZoomLevel;

        /// <summary>
        /// Megjeleníthető kép lekérdezése, vagy beállítása.
        /// </summary>
        public DisplayableImage Image
        {
            set { _Image = value; }
            get { return _Image; }
        }

        public NavigatorControl()
        {
            InitializeComponent();

            _ZoomLevel = 100;
        }

        public void Reset()
        {
            _LabelZoomLevel.Text = "100%";
            _ZoomLevel = 100;
            _TrackZoom.Value = 100;
        }

        private void ButtonRight_Click(object sender, EventArgs e)
        {
            if (_Image != null) _Image.Right();
        }
        private void ButtonLeft_Click(object sender, EventArgs e)
        {
            if (_Image != null) _Image.Left();
        }
        private void ButtonUp_Click(object sender, EventArgs e)
        {
            if (_Image != null) _Image.Up();
        }
        private void ButtonDown_Click(object sender, EventArgs e)
        {
            if (_Image != null) _Image.Down();
        }

        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            if (_ZoomLevel < 1000) _ZoomLevel += 10;
            _TrackZoom.Value = _ZoomLevel;
            _LabelZoomLevel.Text = _ZoomLevel.ToString() + "%";
            if (_Image != null) _Image.ZoomLevel = _ZoomLevel;
        }

        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            if (_ZoomLevel > 11) _ZoomLevel -= 10;
            else if (_ZoomLevel > 1) _ZoomLevel--;
            _TrackZoom.Value = _ZoomLevel;
            _LabelZoomLevel.Text = _ZoomLevel.ToString() + "%";
            if (_Image != null) _Image.ZoomLevel = _ZoomLevel;
        }

        private void TrackZoom_Scroll(object sender, EventArgs e)
        {
            _ZoomLevel = _TrackZoom.Value;
            _LabelZoomLevel.Text = _ZoomLevel.ToString() + "%";
            if (_Image != null) _Image.ZoomLevel = _ZoomLevel;
        }

        private void ButtonReset_Click(object sender, EventArgs e)
        {
            _ZoomLevel = 100;
            _TrackZoom.Value = _ZoomLevel;
            _LabelZoomLevel.Text = _ZoomLevel.ToString() + "%";
            if (_Image != null) _Image.Reset();
        }
    }
}
