﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Shapes = ELTE.Forms.VectorDrawing.Shapes;

namespace ELTE.Forms.VectorDrawing
{
    public partial class DrawingForm : Form
    {
        private Point _StartPoint; // egér hezdőpozíciója a rajzoláshoz
        private List<Shapes.Shape> _Shapes; // alakzatok listája

        public DrawingForm()
        {
            InitializeComponent();

            // panel egérkezelő eseményei
            _Panel.MouseDown += new MouseEventHandler(Panel_MouseDown);
            _Panel.MouseUp += new MouseEventHandler(Panel_MouseUp);
            _Panel.MouseMove += new MouseEventHandler(Panel_MouseMove);

            _Shapes = new List<Shapes.Shape>();
        }

        private void Panel_MouseDown(object sender, MouseEventArgs e)
        {
            _StartPoint = e.Location; // az egér pozícióját eltároljuk
        }

        private void Panel_MouseUp(object sender, MouseEventArgs e)
        {
            // egérfelengedésre hozzávesszük az új alakzatot
            if (_ButtonRectangle.Checked)
            {
                _Shapes.Add(new Shapes.Rectangle(_StartPoint.X < e.X ? _StartPoint.X : e.X, _StartPoint.Y < e.Y ? _StartPoint.Y : e.Y, Math.Abs(e.X - _StartPoint.X), Math.Abs(e.Y - _StartPoint.Y)));
            }
            else if (_ButtonEllipse.Checked)
            {
                _Shapes.Add(new Shapes.Ellipse(_StartPoint.X < e.X ? _StartPoint.X : e.X, _StartPoint.Y < e.Y ? _StartPoint.Y : e.Y, Math.Abs(e.X - _StartPoint.X), Math.Abs(e.Y - _StartPoint.Y)));
            }
            else
            {
                _Shapes.Add(new Shapes.Triangle(_StartPoint.X < e.X ? _StartPoint.X : e.X, _StartPoint.Y < e.Y ? _StartPoint.Y : e.Y, Math.Abs(e.X - _StartPoint.X), Math.Abs(e.Y - _StartPoint.Y)));
            }
            _Panel.Refresh(); // rajzpanel frissítése, kiváltja a Paint eseményt
        }

        private void Panel_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.None) // ha van valamilyen egérgomb lenyomva
            {
                Graphics graphics = _Panel.CreateGraphics();

                graphics.Clear(Color.White); // háttér kitölése fehér színnel
                foreach (var shape in _Shapes)
                {
                    shape.Draw(graphics); // kirajzoljuk az alakzatokat
                }

                // rajzolás alatt álló alakzat kirajzolása:
                Pen pen = new Pen(Color.Blue, 2);
                if (_ButtonRectangle.Checked)
                {
                    graphics.DrawRectangle(pen,
                                           _StartPoint.X < e.X ? _StartPoint.X : e.X, // azt vesszük figyelembe, ami balrább van
                                           _StartPoint.Y < e.Y ? _StartPoint.Y : e.Y,
                                           Math.Abs(e.X - _StartPoint.X), // a méretnél abszolútértéket veszünk
                                           Math.Abs(e.Y - _StartPoint.Y));
                }
                else if (_ButtonEllipse.Checked)
                {
                    graphics.DrawEllipse(pen,
                                         _StartPoint.X < e.X ? _StartPoint.X : e.X, // azt vesszük figyelembe, ami balrább van
                                         _StartPoint.Y < e.Y ? _StartPoint.Y : e.Y,
                                         Math.Abs(e.X - _StartPoint.X), // a méretnél abszolútértéket veszünk
                                         Math.Abs(e.Y - _StartPoint.Y));
                }
                else
                {
                    // mivel álló háromszöveg rajzolunk, mindig alkalmasan választjuk meg a tetejét, illetve az alját
                    Point topLeft = new Point((_StartPoint.X < e.X) ? _StartPoint.X : e.X, (_StartPoint.Y < e.Y) ? _StartPoint.Y : e.Y);
                    Point bottomRight = new Point((_StartPoint.X > e.X) ? _StartPoint.X : e.X, (_StartPoint.Y > e.Y) ? _StartPoint.Y : e.Y);

                    Point[] points = new Point[3]; // a háromszöget poligonként rajzoljuk, pontok segítségével
                    points[0] = new Point(topLeft.X + (bottomRight.X - topLeft.X) / 2, topLeft.Y);
                    points[1] = new Point(topLeft.X, bottomRight.Y);
                    points[2] = new Point(bottomRight.X, bottomRight.Y);

                    graphics.DrawPolygon(pen, points);
                }
            }
        }

        private void ButtonClear_Click(object sender, EventArgs e)
        {
            _Shapes.Clear();
            _Panel.Refresh();
        }

        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            Graphics graphics = e.Graphics; // az eseményargumentumokból is megkapjuk a rajzobjektumot
            graphics.Clear(Color.White); // háttér kitölése fehér színnel
            foreach (var s in _Shapes) // alakzatok kirajzolása
            {
                s.Draw(graphics);
            }
        }
    }
}
